<?xml version="1.0" encoding="UTF-8"?>
<!-- 
  (c) Copyright 2009, Christopher Cormack

  This Software is licensed to You, the licensee, for use under the terms of   
  the 1060 Public License v1.0. Please read and agree to the 1060 Public       
  License v1.0 [www.1060research.com/license] before using or redistributing   
  this software.                                                               

  In summary the 1060 Public license has the following conditions.             
  A. You may use the Software free of charge provided you agree to the terms   
  laid out in the 1060 Public License v1.0                                     
  B. You are only permitted to use the Software with components or applications
  that provide you with OSI Certified Open Source Code [www.opensource.org], or
  for which licensing has been approved by 1060 Research Limited.              
  You may write your own software for execution by this Software provided any  
  distribution of your software with this Software complies with terms set out 
  in section 2 of the 1060 Public License v1.0                                 
  C. You may redistribute the Software provided you comply with the terms of   
  the 1060 Public License v1.0 and that no warranty is implied or given.       
  D. If you find you are unable to comply with this license you may seek to    
  obtain an alternative license from 1060 Research Limited by contacting       
  license@1060research.com or by visiting www.1060research.com                 

  NO WARRANTY:  THIS SOFTWARE IS NOT COVERED BY ANY WARRANTY. SEE 1060 PUBLIC  
  LICENSE V1.0 FOR DETAILS                                                     

  THIS COPYRIGHT NOTICE IS *NOT* THE 1060 PUBLIC LICENSE v1.0. PLEASE READ     
  THE DISTRIBUTED 1060_Public_License.txt OR www.1060research.com/license      

  $Id$
-->
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:tablenk="http://www.onegch.org.uk/netkernel/tablenk"
                xmlns:xs="http://www.w3.org/2001/XMLSchema"
                xmlns="http://www.w3.org/1999/xhtml"
                exclude-result-prefixes="xs tablenk"
                version="2.0">
  <xsl:param name="config" />
  
  <xsl:output indent="yes" method="xml"/>
  
  <xsl:template match="/">
    <table>
      <xsl:if test="$config/config/table/id">
        <xsl:attribute name="id" select="$config/config/table/id"/>
      </xsl:if>
      <xsl:if test="$config/config/table/class">
        <xsl:attribute name="class" select="$config/config/table/class"/>
      </xsl:if>
      <tr>
        <xsl:if test="$config/config/table/headerRow/class">
          <xsl:attribute name="class" select="$config/config/table/headerRow/class"/>
        </xsl:if>
        <xsl:apply-templates select="$config/config/columns/*" mode="header">
          <xsl:with-param name="header" select="//header"/>
        </xsl:apply-templates>
      </tr>
      
      <xsl:choose>
        <xsl:when test="$config/config/table/empty/message and count(//row) = 0">
          <tr>
            <td colspan="{count($config/config/columns/*)}">
              <xsl:if test="$config/config/table/empty/class">
                <xsl:attribute name="class" select="$config/config/table/empty/class"/>
              </xsl:if>
              <xsl:value-of select="$config/config/table/empty/message"/>
            </td>
          </tr>
        </xsl:when>
        <xsl:when test="$config/config/groupings[@mode='extra']">
          <xsl:variable name="base-key" select="$config/config/groupings/base/key"/>
          <xsl:variable name="base-value" select="$config/config/groupings/base/value"/>
          <xsl:variable name="group-key" select="$config/config/groupings/grouping/key"/>
          <xsl:variable name="group-value" select="$config/config/groupings/grouping/value"/>
          <xsl:for-each select="//row">
            <xsl:variable name="level">
              <xsl:choose>
                <xsl:when test="*[local-name()=$base-key][text()=$base-value]">
                  <xsl:text>level0</xsl:text>
                </xsl:when>
                <xsl:otherwise>
                  <xsl:text>level1</xsl:text>
                </xsl:otherwise>
              </xsl:choose>
            </xsl:variable>
            <xsl:variable name="collapsed">
              <xsl:choose>
                <xsl:when test="*[local-name()=$base-key][text()=$base-value]">
                  <xsl:text>false</xsl:text>
                </xsl:when>
                <xsl:otherwise>
                  <xsl:text>true</xsl:text>
                </xsl:otherwise>
              </xsl:choose>
            </xsl:variable>
            
            
            <xsl:variable name="alt">
              <xsl:choose>
                <xsl:when test="$level='level0'">
                  <xsl:value-of select="if (count(preceding-sibling::row[*[local-name()=$base-key][text()=$base-value]]) mod 2 != 0) then ' alt' else ''"/>
                </xsl:when>
                <xsl:otherwise>
                  <!-- see http://www.biglist.com/lists/xsl-list/archives/200602/msg00556.html -->
                  <xsl:value-of select="if (count(for $stop in preceding-sibling::row[*[local-name()=$base-key][text()=$base-value]][1] return preceding-sibling::row[. >> $stop]) mod 2 != 0) then ' alt' else ''"/>
                </xsl:otherwise>
              </xsl:choose>
            </xsl:variable>
            
            <xsl:apply-templates select=".">
              <xsl:with-param name="level" select="$level"/>
              <xsl:with-param name="alt" select="$alt"/>
              <xsl:with-param name="key"> closed</xsl:with-param>
              <xsl:with-param name="collapsed" select="$collapsed"/>
            </xsl:apply-templates>
          </xsl:for-each>
        </xsl:when>
        <xsl:when test="$config/config/groupings/grouping">
          <xsl:variable name="group-key" select="$config/config/groupings/grouping/key"/>
          <xsl:for-each-group select="//row" group-by="*[local-name()=$group-key]/text()">
            <xsl:variable name="key" select="tablenk:string-build(current-group()[1],$config/config/groupings/grouping/class/*)"/>
            <tr class="level0 {$key}">
              <xsl:apply-templates select="$config/config/groupings/grouping/columns/*" mode="group">
                <xsl:with-param name="current-group" select="current-group()"/>
              </xsl:apply-templates>
            </tr>
            <xsl:apply-templates select="current-group()">
              <xsl:with-param name="level" select="'level1'"/>
              <xsl:with-param name="key" select="$key"/>
              <xsl:with-param name="collapsed" select="$config/config/groupings/grouping/collapsed/text()='true'" as="xs:boolean"/>
            </xsl:apply-templates>
          </xsl:for-each-group>
        </xsl:when>
        <xsl:otherwise>
          <xsl:apply-templates select="//row">
            <xsl:with-param name="level"></xsl:with-param>
            <xsl:with-param name="key"></xsl:with-param>
            <xsl:with-param name="alt" select="'pos'"/>
            <xsl:with-param name="collapsed">false</xsl:with-param>
          </xsl:apply-templates>
        </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="//footer"/>
    </table>
  </xsl:template>
  
  <xsl:template match="columns/*[not(self::multicolumn)]" mode="header">
    <xsl:if test="not(type='hidden')">
      <th><xsl:value-of select="title"/></th>
    </xsl:if>
  </xsl:template>
  
  <xsl:template match="columns/multicolumn" mode="header">
    <xsl:param name="header"/>
    <xsl:for-each select="$header/*">
      <th><xsl:copy-of select="."/></th>
    </xsl:for-each>
  </xsl:template>
  
  <xsl:template match="//row">
    <xsl:param name="level"/>
    <xsl:param name="key"/>
    <xsl:param name="alt" select="''"/>
    <xsl:param name="collapsed" as="xs:boolean"/>
    
    <xsl:variable name="newAlt">
      <xsl:choose>
        <xsl:when test="$alt='pos'">
          <xsl:if test="position() mod 2 != 1">
            <xsl:text>alt</xsl:text>
          </xsl:if>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="$alt"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
    
    <tr>
      <xsl:attribute name="class">
        <xsl:value-of select="$level"/>
        <xsl:value-of select="$key"/>
        <xsl:value-of select="$newAlt"/>
      </xsl:attribute>
      <xsl:if test="$collapsed">
        <xsl:attribute name="style" select="'display: none'"/>
      </xsl:if>
      <xsl:apply-templates select="$config/config/columns/*" mode="content">
        <xsl:with-param name="current-row" select="."/>
      </xsl:apply-templates>
    </tr>
  </xsl:template>
  
  <xsl:template match="//footer">
    <tr>
      <xsl:if test="$config/config/table/footerRow/class">
        <xsl:attribute name="class" select="$config/config/table/footerRow/class"/>
      </xsl:if>
      <xsl:apply-templates select="$config/config/columns/*" mode="content">
        <xsl:with-param name="current-row" select="."/>
      </xsl:apply-templates>
    </tr>
  </xsl:template>
  
  <xsl:template match="datacolumn" mode="group">
    <xsl:param name="current-group"/>
    <xsl:variable name="current-config" select="."/>
    <td>
      <xsl:choose>
        <xsl:when test="method='max'">
          <xsl:value-of select="max($current-group//*[local-name()=$current-config/name])"/>
        </xsl:when>
        <xsl:when test="method='avg'">
          <xsl:value-of select="avg($current-group//*[local-name()=$current-config/name])"/>
        </xsl:when>
      </xsl:choose>
    </td>
  </xsl:template>
  
  <xsl:template match="captioncolumn" mode="group">
    <xsl:param name="current-group"/>
    <xsl:variable name="current-config" select="."/>
    <td>
      <xsl:value-of select="tablenk:string-build(current-group()[1],caption/*)"/>
    </td>
  </xsl:template>
  
  <xsl:template match="emptycolumn" mode="group">
    <td></td>
  </xsl:template>
  
  <xsl:template match="columns/datacolumn" mode="content">
    <xsl:param name="current-row"/>
    <xsl:variable name="current-config" select="."/>
    <xsl:variable name="value" select="$current-row/*[local-name()=$current-config/name]"/>
    <td>
      <xsl:apply-templates select="id,class">
        <xsl:with-param name="current-row" select="$current-row"/>
      </xsl:apply-templates>
      <xsl:choose>
        <xsl:when test="not($current-config/type)">
          <xsl:copy-of select="$value/node()"/>
        </xsl:when>
        <xsl:when test="$current-config/type='currency'">
          <xsl:value-of select="$value"/>
        </xsl:when>
        <xsl:when test="$current-config/type='date'">
          <xsl:choose>
            <xsl:when test="$value/*">
              <xsl:apply-templates select="$value/*" mode="date"/>
            </xsl:when>
            <xsl:otherwise>
              <xsl:apply-templates select="$value/text()" mode="date"/>
            </xsl:otherwise>
          </xsl:choose>
        </xsl:when>
        <xsl:when test="$current-config/type='hidden'">
          <xsl:attribute name="style" select="'display: none;'"/>
          <xsl:value-of select="$value"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:message>TableNK: unknown type <xsl:value-of select="$current-config/type"/></xsl:message>
          <xsl:value-of select="$value"/>
        </xsl:otherwise>
      </xsl:choose>
    </td>
  </xsl:template>
  
  <xsl:template match="*" mode="date">
    <xsl:copy>
      <xsl:apply-templates select="@* | node()" mode="#current"/>
    </xsl:copy>
  </xsl:template>
  
  <xsl:template match="text()" mode="date">
    <xsl:value-of select="format-date(., '[D01]/[M01]/[Y0001]')"/>
  </xsl:template>
  
  <xsl:template match="columns/multicolumn" mode="content">
    <xsl:param name="current-row"/>
    <xsl:variable name="current-config" select="."/>
    <xsl:variable name="value" select="$current-row/*[local-name()=$current-config/name]"/>
    <xsl:for-each select="$value">
      <td>
      <xsl:apply-templates select="id,class">
        <xsl:with-param name="current-row" select="$current-row"/>
      </xsl:apply-templates>
      <xsl:choose>
        <xsl:when test="not($current-config/type)">
          <xsl:copy-of select="."/>
        </xsl:when>
        <xsl:when test="$current-config/type='currency'">
          <xsl:value-of select="."/>
        </xsl:when>
        <xsl:when test="$current-config/type='date'">
          <xsl:value-of select="if (. != '') then format-date(., '[D01]/[M01]/[Y0001]') else ''"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:message>TableNK: unknown type <xsl:value-of select="$current-config/type"/></xsl:message>
          <xsl:value-of select="."/>
        </xsl:otherwise>
      </xsl:choose>
    </td>
    </xsl:for-each>
  </xsl:template>
  
  <xsl:template match="columns/controlcolumn" mode="content">
    <xsl:param name="current-row"/>
    <xsl:variable name="current-config" select="."/>
    <xsl:variable name="value" select="$current-row/*[local-name()=$current-config/name]" as="xs:string"/>
    <xsl:variable name="caption" select="tablenk:string-build($current-row, $current-config/caption/*)"/>
    <td>
      <xsl:if test="not($current-config/depends/node) or
                    $current-row/*[local-name()=$current-config/depends/node and
                                   text()=$current-config/depends/value]">
        <xsl:apply-templates select="id,class">
          <xsl:with-param name="current-row" select="$current-row"/>
        </xsl:apply-templates>
        <xsl:choose>
          <xsl:when test="$current-config/type='link'">
            <xsl:variable name="uri" select="tablenk:string-build($current-row, $current-config/uri/*)"/>
            <a href="{$uri}"><xsl:value-of select="$caption"/></a>
          </xsl:when>
          <xsl:otherwise>
            
          </xsl:otherwise>
        </xsl:choose>
      </xsl:if>
    </td>
  </xsl:template>
  
  <xsl:template match="columns/inputcolumn" mode="content">
    <xsl:param name="current-row"/>
    <xsl:variable name="current-config" select="."/>
    <xsl:variable name="value" select="$current-row/*[local-name()=$current-config/name]" as="xs:string"/>
    <td>
      <xsl:if test="not($current-config/depends/node) or
                    $current-row/*[local-name()=$current-config/depends/node and
                                   text()=$current-config/depends/value]">
        <xsl:apply-templates select="id,class">
          <xsl:with-param name="current-row" select="$current-row"/>
        </xsl:apply-templates>
        <xsl:choose>
          <xsl:when test="input/type='checkbox'">
            <input type="checkbox">
              <xsl:if test="input/checked">
                <xsl:attribute name="checked" select="'checked'"/>
              </xsl:if>
              <xsl:apply-templates select="input/id,input/name,input/class">
                <xsl:with-param name="current-row" select="$current-row"/>
              </xsl:apply-templates>
            </input>
          </xsl:when>
          <xsl:otherwise>
            
          </xsl:otherwise>
        </xsl:choose>
      </xsl:if>
    </td>
  </xsl:template>
  
  <xsl:template match="id">
    <xsl:param name="current-row"/>
    <xsl:variable name="id" select="tablenk:string-build($current-row, ./*)"/>
    <xsl:attribute name="id" select="$id"/>
  </xsl:template>
  
  <xsl:template match="name">
    <xsl:param name="current-row"/>
    <xsl:variable name="name" select="tablenk:string-build($current-row, ./*)"/>
    <xsl:attribute name="name" select="$name"/>
  </xsl:template>
  
  <xsl:template match="class">
    <xsl:param name="current-row"/>
    <xsl:variable name="class" select="tablenk:string-build($current-row, ./*)"/>
    <xsl:attribute name="class" select="$class"/>
  </xsl:template>
  
  <xsl:function name="tablenk:string-build" as="xs:string">
    <xsl:param name="current-row" as="node()"/>
    <xsl:param name="parts" as="node()*"/>
    
    <xsl:variable name="values" as="node()*">
      <xsl:for-each select="$parts">
        <value>
          <xsl:choose>
            <xsl:when test="self::string">
              <xsl:value-of select="text()"/>
            </xsl:when>
            <xsl:when test="self::datafield">
              <xsl:variable name="field" select="text()"/>
              <xsl:value-of select="$current-row/*[local-name()=$field]"/>
            </xsl:when>
            <xsl:otherwise>
              <xsl:message>TableNK: Unknown child for string-build <xsl:value-of select="local-name()"/></xsl:message>
            </xsl:otherwise>
          </xsl:choose>
        </value>
      </xsl:for-each>
    </xsl:variable>
    
    <xsl:value-of select="string-join($values//text(),'')"/>
  </xsl:function>
</xsl:stylesheet>
